/*
 * (C) Copyright ParaSoft Corporation 2022.  All rights reserved.
 * THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF ParaSoft
 * The copyright notice above does not evidence any
 * actual or intended publication of such source code.
 */

#include "Logger.h"
#include "Environment.h"

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <wchar.h>

Logger::Logger()
{
    _debugFile = Environment::getenv("PARASOFT_DEBUG_FILE");
    if (!_debugFile || !*_debugFile) {
        _debugFile = Environment::getenv("PARASOFT_DEBUG_NET_INTERFACES_FILE");
    }

    const char* isDebugStr = Environment::getenv("PARASOFT_DEBUG");
    const char* isLegacyDebugStr = Environment::getenv("PARASOFT_DEBUG_NET_INTERFACES");

    _isDebug = 
        (_debugFile && *_debugFile) || 
        ((isDebugStr != 0) && (strcmp(isDebugStr, "true") == 0)) || 
        ((isLegacyDebugStr != 0) && (strcmp(isLegacyDebugStr, "true") == 0));
}


Logger& Logger::getLogger()
{
    static Logger LOGGER;
    return LOGGER;
}


void Logger::debug(const char* format, ...)
{
    if (!_isDebug) {
        return;
    }

    va_list argptr;    va_start(argptr, format);

    if (_debugFile && *_debugFile) {
        FILE* debugFile = fopen(_debugFile, "a");
        if (debugFile != NULL) {
            vfprintf(debugFile, format, argptr);
            fclose(debugFile);
        } else {
            printf("[Wrong PARASOFT_DEBUG_FILE] %s\n", _debugFile);
            vfprintf(stdout, format, argptr);
        }
    } else {
        vfprintf(stdout, format, argptr);
    }

    va_end(argptr);
}

void Logger::wdebug(const wchar_t* format, ...)
{
    if (!_isDebug) {
        return;
    }

    va_list argptr;    va_start(argptr, format);

    if (_debugFile && *_debugFile) {
        FILE* debugFile = fopen(_debugFile, "a");
        if (debugFile != NULL) {
            vfwprintf(debugFile, format, argptr);
            fclose(debugFile);
        } else {
            printf("[Wrong PARASOFT_DEBUG_FILE] %s\n", _debugFile);
            vfwprintf(stdout, format, argptr);
        }
    } else {
        vfwprintf(stdout, format, argptr);
    }

    va_end(argptr);
}
