#include "MachineId.h"
#include <stdio.h>

#include <stdlib.h>
#include <assert.h>

#if defined(__linux__) || defined(_WIN32)
class MacBasedMachineIdTest
{
public:

    void testCompareInterfaces()
    {
        MacBasedMachineId id;

        InterfaceInfo iInfo1;
        InterfaceInfo iInfo2;

        // Custom interface wins
        initializeInterfaceInfo(iInfo1, 2, 2, "", L"", 0, true, false, true, 1, 1);
        initializeInterfaceInfo(iInfo2, 0, 0, "", L"", 0, false, true, false, 2, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);

        // Interface with mac address wins
        initializeInterfaceInfo(iInfo1, 0, 0, "", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "", L"", 0, false, false, true, 0, 1);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);

        // Better quality wins
        initializeInterfaceInfo(iInfo1, 0, 1, "", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 1, 0, "", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);

        // Better prefix wins
        initializeInterfaceInfo(iInfo1, 0, 0, "enp1", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "eth1", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);
        initializeInterfaceInfo(iInfo1, 0, 0, "enp1", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "eth0", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);

        // Better suffix wins
        initializeInterfaceInfo(iInfo1, 0, 0, "eth0", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "eth1", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);
        initializeInterfaceInfo(iInfo1, 0, 0, "eth", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "eth0", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);
        initializeInterfaceInfo(iInfo1, 0, 0, "eth0", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "eth0:1", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);

        // no prefix - lexical orderwins
        initializeInterfaceInfo(iInfo1, 0, 0, "aaaa", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "bbb", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);

        // If all is the same, AF_INET wins with AF_INET6
        initializeInterfaceInfo(iInfo1, 0, 0, "a", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 1, "a", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);
    }

#if 0
    void testEnv()
    {
        setenv("PARASOFT_SUPPORT_NET_INTERFACES", "int1", 1);
        MachineId id;

        InterfaceInfo iInfo1;
        InterfaceInfo iInfo2;
        initializeInterfaceInfo(iInfo1, 0, 0, "int1", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "enp0", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);

        initializeInterfaceInfo(iInfo1, 0, 0, "int1:0", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "enp0", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);
    }

    void testEnv2()
    {
        // Spaces are not allowed
        setenv("PARASOFT_SUPPORT_NET_INTERFACES", "int3,int2,int1", 1);
        MachineId id;

        InterfaceInfo iInfo1;
        InterfaceInfo iInfo2;
        initializeInterfaceInfo(iInfo1, 0, 0, "int2", L"", 0, false, true, false, 1, 0);
        initializeInterfaceInfo(iInfo2, 0, 0, "int1", L"", 0, false, true, false, 1, 0);
        assert(id.compareInterfaces(iInfo1, iInfo2) == MachineId::BETTER);
        assert(id.compareInterfaces(iInfo2, iInfo1) == MachineId::WORSE);
    }
#endif

    void initializeInterfaceInfo(InterfaceInfo& iInfo,
        int quality, int lowQuality, const char* name, const wchar_t* wname, const char* prefix,
        bool isCustom, bool hasMac, bool hasIp, unsigned char mac0, unsigned char ip0)
    {
        iInfo.quality = quality;
        iInfo.lowQuality = lowQuality;
#if defined(__linux__)
        iInfo.name = name ? strdup(name) : (char*)0;
        iInfo.wname = wname ? wcsdup(wname) : (wchar_t*)0;
#else
        iInfo.name = name ? _strdup(name) : (char*)0;
        iInfo.wname = wname ? _wcsdup(wname) : (wchar_t*)0;
#endif
        iInfo.prefix = prefix;
        iInfo.isCustom = isCustom;
        iInfo.hasMac = hasMac;
        iInfo.hasIp = hasIp;
        iInfo.mac[0] = mac0;
        iInfo.ip[0] = ip0;
    }
};
#endif

int main()
{
#if defined(__linux__) || defined(_WIN32)
    MacBasedMachineIdTest test;
    test.testCompareInterfaces();
    //test.testEnv();
    //test.testEnv2();
#endif

    MachineId id;
    printf("Machine id: %x\n", id.getMachineId());
    return 0;
}
