<?xml version="1.0"?>
<!-- in case of any change in xsl/xml file modify all connected xsl and xml files from this package  -->
<xsl:stylesheet version="1.1" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:saxon="http://icl.com/saxon" xmlns:graph="graph" xmlns:results="results" extension-element-prefixes="saxon">
    
    <!-- ================ print functional menu ======================= -->
    <xsl:template name="display_functional_menu">
        <xsl:param name="isFunctional"/>
       	<xsl:param name="auth"/>
           
	    <xsl:variable name="href" saxon:assignable="yes"/>
	    <xsl:if test="not($auth)">
	    	<xsl:choose>
                 <xsl:when test="($isFunctional) = 'true'">
                     <saxon:assign name="href">#func</saxon:assign>
                 </xsl:when>
                 <xsl:otherwise>
                     <saxon:assign name="href">#change</saxon:assign>
                 </xsl:otherwise>
             </xsl:choose>
	    </xsl:if>
	    <xsl:if test="$auth">
	    	<xsl:choose>
                 <xsl:when test="($isFunctional) = 'true'">
                 	 <saxon:assign name="href" select="concat('#func_',$auth)"/>
                 </xsl:when>
                 <xsl:otherwise>
                     <saxon:assign name="href" select="concat('#change_',$auth)"/>
                 </xsl:otherwise>
             </xsl:choose>
	    </xsl:if>
        
        
        <xsl:if test="(($functional > 0) and ($isFunctional = 'true')) or (($change_advisor > 0) and ($isFunctional = 'false'))">
                    <xsl:variable name="caption" saxon:assignable="yes"/>
                    <xsl:choose>
                        <xsl:when test="($isFunctional) = 'false'">
                        	<saxon:assign name="caption" select="$RES_PARAMETER_CHANGE_IMPACT" />
                        </xsl:when>
                        <xsl:when test="($toolName='Virtualize')">
                        	<saxon:assign name="caption" select="$RES_PARAMETER_PROVISIONING_ACTIONS"/>
                        </xsl:when>
                        <xsl:otherwise>
                        	<saxon:assign name="caption" select="$RES_PARAMETER_FUNCTIONAL_TESTS"/>
                        </xsl:otherwise>
                    </xsl:choose>
			        <xsl:call-template name="display_linked_menu_item">
				  	 <xsl:with-param name="text" select="$caption" />
				     <xsl:with-param name="link" select="$href" />
				    </xsl:call-template>	    	                      
                
        </xsl:if>
    </xsl:template>

    <!-- =================  print functional header ============================ -->
    <xsl:template name="func_header">
        <xsl:param name="ref" />
        <xsl:param name="isFunctional" />
        <xsl:if test="not(string-length($ref) > 0)">
            <xsl:variable name="anchor_ref" saxon:assignable="yes"/>
            <xsl:choose>
                <xsl:when test="($isFunctional) = 'true'">
                    <saxon:assign name="anchor_ref">func</saxon:assign>
                </xsl:when>
                <xsl:otherwise>
                    <saxon:assign name="anchor_ref">change</saxon:assign>
                </xsl:otherwise>
            </xsl:choose>
            <a name="{$anchor_ref}" />
        </xsl:if>
        <xsl:if test="string-length($ref) > 0">
            <a name="{$ref}" />
        </xsl:if>
        <table width="{$width}"  border="0"
            bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0">
            <xsl:if test="($report_format_mode='pdf')">
               <xsl:attribute name="cols">15% 70% 15%</xsl:attribute>
            </xsl:if>
            <tr>
                <td width="15%"></td>
                <td align="center">                    
                   <div class="header">
                       <xsl:choose>
                           <xsl:when test="($isFunctional) = 'false'">
                               <xsl:value-of select="$RES_PARAMETER_CHANGE_IMPACT"/>
                           </xsl:when>
                           <xsl:when test="($toolName='Virtualize')">
                               <xsl:value-of select="$RES_PARAMETER_PROVISIONING_ACTIONS"/>
                           </xsl:when>
                           <xsl:otherwise>
                               <xsl:value-of select="$RES_PARAMETER_FUNCTIONAL_TESTS" />
                           </xsl:otherwise>
                       </xsl:choose>
                   </div>
                </td>
                <td width="15%" align="right" valign="bottom">
	  	           <xsl:if test="not($report_type = $email_report) and ($first_back_to_top = 'false')">
                      <xsl:call-template name="back_to_top"/>
                   </xsl:if>
   		           <saxon:assign name="first_back_to_top">false</saxon:assign>
                </td>
            </tr>
        </table>
    </xsl:template>

    <!-- =================  func_authors_summary ============================ -->
    <xsl:template name="func_authors_summary">
        <xsl:param name="isFunctional"/>
    	<xsl:call-template name="authors_header"/>
        <table align="center" class="singleborder" width="{$authors_width}" bordercolor='{$func_bdrcolor}' cellpadding='0' cellspacing='0'>
            <xsl:if test="($report_format_mode='pdf')">
                <xsl:attribute name="cols">50% 25% 25%</xsl:attribute>
            </xsl:if>
            <tr bgcolor="{$func_bcolor}">
                <td class="singleborder" width="50%" align="center">
                    <b><xsl:value-of select="$RES_PARAMETER_AUTHOR" /></b>
                </td>
                <td class="singleborder" width="25%" align="center">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_TASKS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_CHANGES" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                    <!--  NYI These options are not yet supported
                        <xsl:value-of select="$RES_PARAMETER_SUPPRESSED"/>
                        <xsl:value-of select="$RES_PARAMETER_TOTAL_RECOMMENDED"/> -->
                </td>
                <td class="singleborder" width="25%" align="center">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_FAILED_TESTS"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_CHANGED_TESTS"/>
                        </xsl:otherwise>
                    </xsl:choose></b>
                        
                </td>
            </tr>
            <xsl:call-template name="func_author_list">
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </table>
    </xsl:template>
    
    <!-- ===================== get complete name for test ========= -->
    <xsl:template name="func_test_name">
        <xsl:param name="includePath"/>
        <xsl:param name="name"/>
        <xsl:param name="testID"/>
        <xsl:param name="testNode"/>
        <xsl:choose>
            <xsl:when test="($includePath)">
                <xsl:call-template name="parent_path_to_test">
                    <xsl:with-param name="dirty_path" select="($testID)" />
                </xsl:call-template>
                <xsl:variable name="pathToTest">
                    <xsl:call-template name="path_to_test">
                        <xsl:with-param name="current_node" select="($testNode)"/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:if test="not(($pathToTest) = '')">
                    - <xsl:value-of select="($pathToTest)"/>
                </xsl:if>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="($name)"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- ===================== get parent path for test =========== -->
    <xsl:template name="parent_path_to_test">
        <xsl:param name="dirty_path"/>
        <xsl:variable name="trim_start" select="substring-after(($dirty_path),':///')"/>
        
        <xsl:variable name="clean_path" select="substring-before(($trim_start),'#')" saxon:assignable="yes"/>

        <xsl:if test="not(string-length($clean_path) > 0)">
            <saxon:assign name="clean_path" select="substring-before(($trim_start),'.')"/>
        </xsl:if>
        <xsl:value-of select="($clean_path)"/>
    </xsl:template>

    <!-- =================  get path to test ================== -->
    <xsl:template name="path_to_test">
        <xsl:param name="current_node"/>
        <xsl:variable name="path" saxon:assignable="yes"/>
        
        <xsl:for-each select="($current_node)">
            <xsl:if test="(name() = 'TestSuite' or name() = 'Test') and contains(@id, '#')">
                <xsl:variable name="parent_path">
                    <xsl:call-template name="path_to_test">
                        <xsl:with-param name="current_node" select=".."/>
                    </xsl:call-template>
                </xsl:variable>
                <xsl:choose>
                    <xsl:when test="($parent_path) = ''">
                        <saxon:assign name="path" select="@name"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="path" select="concat(($parent_path),'/',@name)"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:if>
        </xsl:for-each>
        <xsl:value-of select="($path)"/>
    </xsl:template>
    
    <!-- =================  get total failed tests for a test suite for an author ====== -->
    <xsl:template name="total_failed_sub_tests">
        <xsl:param name="suite"/>
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>
        
        <xsl:variable name="success">
            <xsl:call-template name="total_success_sub_tests">
                <xsl:with-param name="suite" select="($suite)"/>
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="total">
            <xsl:call-template name="total_sub_tests">
                <xsl:with-param name="suite" select="($suite)"/>
                <xsl:with-param name="isFunctional" select="$isFunctional"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="failed" select="($total) - ($success)"/>
        <xsl:value-of select="($failed)"/>
    </xsl:template>
    
    <!-- =================  get total tests for a test suite  =============== -->
    <xsl:template name="total_sub_tests">
        <xsl:param name="suite"/>
        <xsl:param name="isFunctional"/>
        
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="test_count" select="$suite/@total"/>
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="test_count" select="$suite/@changeTotal"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:value-of select="($test_count)"/>
    </xsl:template>

    <!-- =================  get total succeeded tests for a test suite for an author ======= -->
    <xsl:template name="total_success_sub_tests">
        <xsl:param name="suite"/>
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>
    
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($auth) = ''">
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <saxon:assign name="test_count" select="$suite/@pass"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="test_count" select="$suite/@changePass"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
            <!-- not implemented -->
               <saxon:assign name="test_count" select="-20"/>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:value-of select="($test_count)"/>
    </xsl:template>
    
    <!-- =================  get total tests ================== -->
    <xsl:template name="total_tests">
        <xsl:param name="isFunctional"/>
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="test_count" select="/ResultsSession/ExecutedTestsDetails/Total/@total"/>
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="test_count" select="/ResultsSession/ExecutedTestsDetails/Total/@changeTotal"/>
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:value-of select="($test_count)"/>
    </xsl:template>
    
    <!-- =================  get total failed tests for an author =========== -->
    <xsl:template name="total_failed_tests">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>
    
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($auth) = ''">
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <saxon:assign name="test_count" select="/ResultsSession/ExecutedTestsDetails/Total/@fail"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="test_count" select="/ResultsSession/ExecutedTestsDetails/Total/@change"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
            <!-- not implemented -->
               <saxon:assign name="test_count" select="-20"/>
            </xsl:otherwise>
        </xsl:choose>

        <xsl:value-of select="($test_count)"/>
    </xsl:template>
    
    <!-- =================  get total successful tests for an author ======= -->
    <xsl:template name="total_successful_tests">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>
        
        <xsl:variable name="test_count" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($auth) = ''">
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <saxon:assign name="test_count" select="/ResultsSession/ExecutedTestsDetails/Total/@pass"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="test_count" select="/ResultsSession/ExecutedTestsDetails/Total/@changePass"/>
                    </xsl:otherwise>
                </xsl:choose>
            </xsl:when>
            <xsl:otherwise>
            <!-- not implemented -->
               <saxon:assign name="test_count" select="-20"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:value-of select="($test_count)"/>
    </xsl:template>
    
    <!-- =================  print functional test summary info ============= -->
    <xsl:template name="func_tests_summary">
        <xsl:param name="auth"/>
        <xsl:param name="isFunctional"/>
        
        <xsl:variable name="totalTests">
            <xsl:call-template name="total_tests">
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="successfulTests">
            <xsl:call-template name="total_successful_tests">
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="failedTests">
            <xsl:call-template name="total_failed_tests">
                <xsl:with-param name="auth" select="($auth)"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:variable>
        <xsl:variable name="numberRootTestSuites" select="count(/ResultsSession/ExecutedTestsDetails/Total/Project//*[@root = 'true'])"/>
        <xsl:variable name="background">
            <xsl:choose>
                <xsl:when test="($failedTests) > 0">errorsingleborder</xsl:when>
                <xsl:otherwise>singleborder</xsl:otherwise>
            </xsl:choose>
        </xsl:variable>
        <xsl:call-template name="summary_header">
            <xsl:with-param name="isFunctional" select="($isFunctional)"/>
        </xsl:call-template>
        <table class="singleborder" width="{$width}" bordercolor="{$func_bdrcolor}"
            cellpadding="0" cellspacing="0">
            <xsl:if test="($report_format_mode='pdf')">                
                <xsl:if test="($test_suites_only) = 'true'">
                    <xsl:attribute name="cols">48% 13% 13% 13% 13%</xsl:attribute>
                </xsl:if>
                <xsl:if test="($test_suites_only) = 'false'">
                    <xsl:attribute name="cols">45% 11% 11% 11% 11% 11%</xsl:attribute>
                </xsl:if>
            </xsl:if>
            <tr>
                <td class="singleborder" bgcolor="{$func_bcolor}" width="45%" align="center">
                    <xsl:choose>
                        <xsl:when test="($test_suites_only) = 'false'">
                            <b><xsl:choose>
                                <xsl:when test="($isFunctional) = 'true'">
                                    <xsl:value-of select="$RES_PARAMETER_NAME" />
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="$RES_PARAMETER_IMPACTED" />
                                </xsl:otherwise>
                            </xsl:choose></b>
                        </xsl:when>
                        <xsl:otherwise>
                            <b><xsl:value-of select="$RES_PARAMETER_NAME2" /></b>
                        </xsl:otherwise>
                    </xsl:choose>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_FAILED_TESTS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_CHANGED_TESTS" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_SUCCESSFUL_TESTS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_UNCHANGED_TESTS" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_TESTS" />
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_ASSETS" />
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                    <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_SUCCESS_PERCENT"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_IMPACT_PERCENT"/>
                        </xsl:otherwise>
                    </xsl:choose></b>
                </td>
                <xsl:if test="($test_suites_only) = 'false'">
                    <td class="singleborder" bgcolor="{$func_bcolor}" align="center" width="11%">
                        <b><xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_TASKS"/>
                        </xsl:when>
                        <xsl:otherwise>
                            <xsl:value-of select="$RES_PARAMETER_TOTAL_CHANGES"/>
                        </xsl:otherwise>
                    </xsl:choose></b>
                    </td>
                </xsl:if>
            </tr>
            <xsl:if test="($numberRootTestSuites > 1) or ($test_suites_only = 'false')">
                <tr>
                    <td align="left" width="45%" class="{$background}">
                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                        <b><xsl:value-of select="$RES_PARAMETER_TOTAL"/></b>
                    </td>
                    <td width="11%" align="right" class="{$background}">
                        <xsl:value-of select="$failedTests" /><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <td width="11%" align="right" class="{$background}">
                        <xsl:value-of select="$successfulTests" /><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <td width="11%" align="right" class="{$background}">
                        <xsl:value-of select="$totalTests"/><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <td width="11%" align="right" class="{$background}">
                        <xsl:variable name="numerator">
                            <xsl:choose>
                                <xsl:when test="($isFunctional) = 'true'">
                                    <xsl:value-of select="($successfulTests)"/>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="($failedTests)"/>
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <xsl:call-template name="get_percent">
                            <xsl:with-param name="numerator" select="($numerator)"/>
                            <xsl:with-param name="denominator" select="($totalTests)"/>
                        </xsl:call-template>
                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                    </td>
                    <xsl:if test="($test_suites_only) = 'false'">
                        <td width="11%" align="right" class="{$background}">
                            <xsl:choose>
                                <xsl:when test="($auth) = ''">
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/FunctionalTests/FuncViols/*[not(@supp='true')])"/>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[not(@supp='true')])"/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/FunctionalTests/FuncViols/*[@auth=($auth) and not(@supp='true')])"/>
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@auth=($auth) and not(@supp='true')])"/>
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:otherwise>
                            </xsl:choose>
                            <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                        </td>
                    </xsl:if>
                </tr>
            </xsl:if>
            <xsl:variable name="hasTestsWithAssociations" select="count(/ResultsSession/ExecutedTestsDetails//Test/assoc) > 0"/>
            <xsl:variable name="hasTestsWithComments" select="count(/ResultsSession/ExecutedTestsDetails//Test[@comment != '']) > 0"/>
            <xsl:variable name="hasTestsWithScreenshots" select="count(/ResultsSession/ExecutedTestsDetails//Test//Screenshot) > 0"/>
            <xsl:variable name="testsHaveDetails" select="(($hasTestsWithAssociations) or ($hasTestsWithComments) or ($hasTestsWithScreenshots)) and (($isFunctional) = 'true')" />
            
            <xsl:variable name="hasFailedTestsWithAssociations" select="count(/ResultsSession/ExecutedTestsDetails//Test[@fail != '0']/assoc) > 0"/>
            <xsl:variable name="hasFailedTestsWithComments" select="count(/ResultsSession/ExecutedTestsDetails//Test[@comment != '' and @fail != '0']) > 0"/>
            <xsl:variable name="hasFailedTestsWithScreenshots" select="count(/ResultsSession/ExecutedTestsDetails//Test[@fail != '0']//Screenshot) > 0"/>
            <xsl:variable name="failedTestsHaveDetails" select="($hasFailedTestsWithAssociations) or ($hasFailedTestsWithComments) or ($hasFailedTestsWithScreenshots)" />
            
            <xsl:variable name="hasTestsToShow" select="(($failed_tests_only) = 'false') or ($failedTestsHaveDetails)" />
            <xsl:variable name="useExpandCollapse" select="($testsHaveDetails) and (($test_suites_only) = 'false') and ($hasTestsToShow) and (not(($report_type) = ($email_report))) and (not($report_format_mode='pdf'))" />
            <xsl:variable name="topTest" select="/ResultsSession/ExecutedTestsDetails/Total/Project//*[@root = 'true']" />

	    <xsl:choose>
            <xsl:when test="($test_suites_only) = 'false'">
            <tr>
                <td align="left" class="singleborder" style="padding-left:5px;">
                <xsl:if test="($test_suites_only) = 'true'">
                    <xsl:attribute name="colspan">5</xsl:attribute>
                </xsl:if>
                <xsl:if test="($test_suites_only) = 'false'">
                    <xsl:attribute name="colspan">6</xsl:attribute>
                </xsl:if>
                <table width="{$width}" bordercolor="{$func_bdrcolor}" cellpadding="0" cellspacing="0">
                    <xsl:if test="($report_format_mode='pdf')">                
                        <xsl:attribute name="cols">89% 11%</xsl:attribute>
                    </xsl:if>
                    <xsl:if test="($useExpandCollapse)">
                      <tr>
                        <td class="collapse_controls" colspan="2">
                            <a onmouseover="this.style.cursor='pointer'" onclick="expandCollapseAllDetails(false)"><xsl:value-of select="($RES_PARAMETER_COLLAPSE_ALL)" /></a>
                            <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                            <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                            <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                            <a onmouseover="this.style.cursor='pointer'" onclick="expandCollapseAllDetails(true)"><xsl:value-of select="($RES_PARAMETER_EXPAND_ALL)" /></a>
                            <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                         </td>
                      </tr>
                    </xsl:if>
                    <xsl:for-each select="($topTest)">
                        <xsl:call-template name="func_test_info">
                            <xsl:with-param name="test" select="." />
                            <xsl:with-param name="testID" select="./@id" />
                            <xsl:with-param name="depth">0</xsl:with-param>
                            <xsl:with-param name="auth" select="($auth)"/>
                            <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                        </xsl:call-template>
                    </xsl:for-each>
		</table>
	      </td>
            </tr>
            </xsl:when>
            <xsl:otherwise>
                <xsl:for-each select="($topTest)">
                    <xsl:call-template name="func_test_info">
                        <xsl:with-param name="test" select="." />
                        <xsl:with-param name="testID" select="./@id" />
                        <xsl:with-param name="depth">0</xsl:with-param>
                        <xsl:with-param name="auth" select="($auth)"/>
                        <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                    </xsl:call-template>
                </xsl:for-each>
	    </xsl:otherwise>
            </xsl:choose>
        </table>
    </xsl:template>

    <!-- ================= summary_header ============================ -->
    <xsl:template name="summary_header">
        <xsl:param name="isFunctional"/>
    
    <table width='{$width}' border='0' cellpadding='0' cellspacing='0'>
    <xsl:if test="($report_format_mode='pdf')">
       <xsl:attribute name="cols">100%</xsl:attribute>
    </xsl:if>
    <tr>
        <td align="center"><div class="small_header">
            <xsl:choose>
                <xsl:when test="($isFunctional) = 'false'">
                    <xsl:value-of select="$RES_PARAMETER_ASSET_SUMMARY"/>
                </xsl:when>
                <xsl:when test="($toolName='Virtualize')">
                    <xsl:value-of select="$RES_PARAMETER_ACTION_SUITE_SUMMARY"/>
                </xsl:when>
                <xsl:otherwise>
                    <xsl:value-of select="$RES_PARAMETER_TEST_SUITE_SUMMARY" />
                </xsl:otherwise>
            </xsl:choose>
        </div></td>
    </tr>
    </table>    
        
    </xsl:template>

    <!-- =================  print a tests information  ============================ -->
    <xsl:template name="func_test_info">
        <xsl:param name="test" />
        <xsl:param name="testID" />
        <xsl:param name="depth" />
        <xsl:param name="auth" />
        <xsl:param name="isFunctional" />

        <xsl:variable name="width" select="($depth) * 20" />
        <xsl:variable name="widthPX" select="concat(($width),'px')" />
        <xsl:variable name="background" saxon:assignable="yes">singleborder</xsl:variable>
        <xsl:if test="($test_suites_only) = 'false'">    
           <saxon:assign name="background">no_error</saxon:assign>
        </xsl:if>
	<xsl:variable name="fontColor" saxon:assignable="yes">green</xsl:variable>
        
        <xsl:variable name="hasDetails" select="(($associations = 'true' and (count(./assoc) > 0 or (./@comment != ''))) or (count(.//Screenshot) > 0)) and (($isFunctional) = 'true')" />
        <xsl:variable name="detailsID" select="concat(($testID), '_details')" />
        
        <xsl:variable name="isFailed" saxon:assignable="yes">0</xsl:variable>
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="isFailed" select="./@fail"/>
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="isFailed" select="./@change"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="($isFailed) > 0">
            <xsl:if test="($failed_tests_only) = 'false'">
                <xsl:choose>
                    <xsl:when test="($test_suites_only) = 'false'">    
                        <saxon:assign name="background">error</saxon:assign>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="background">errorsingleborder</saxon:assign>
                    </xsl:otherwise>
	            </xsl:choose>
	        </xsl:if>
            <saxon:assign name="fontColor">red</saxon:assign>
        </xsl:if>
        <xsl:variable name="isSuite" saxon:assignable="yes"></xsl:variable>
        <xsl:for-each select="($test)">
            <saxon:assign name="isSuite" select="name() = 'TestSuite'" />
        </xsl:for-each>
        <xsl:choose>
            <xsl:when test="($isSuite) > 0 or ($depth) = 0">
                <xsl:for-each select="($test)">
                    <xsl:variable name="test_name">
                        <xsl:choose>
                            <xsl:when test="($depth) = 0 and ($isSuite) > 0 and ($test_suites_only) = 'true'">
                                <xsl:call-template name="parent_path_to_test">
                                    <xsl:with-param name="dirty_path" select="($testID)" />
                                </xsl:call-template>
                            </xsl:when>
                            <xsl:otherwise>
                                <xsl:call-template name="func_test_name">
                                    <xsl:with-param name="includePath" select="($depth) = 0"/>
                                    <xsl:with-param name="name" select="@name"/>
                                    <xsl:with-param name="testID" select="($testID)"/>
                                    <xsl:with-param name="testNode" select="."/>
                                </xsl:call-template>
                            </xsl:otherwise>
                        </xsl:choose>
                    </xsl:variable>
                    <xsl:variable name="totalSuccess">
                        <xsl:call-template name="total_success_sub_tests">
                            <xsl:with-param name="suite" select="." />
                            <xsl:with-param name="auth" select="($auth)" />
                            <xsl:with-param name="isFunctional" select="($isFunctional)" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="totalTests">
                        <xsl:call-template name="total_sub_tests">
                            <xsl:with-param name="suite" select="." />
                            <xsl:with-param name="isFunctional" select="($isFunctional)" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="totalFailed">
                        <xsl:call-template name="total_failed_sub_tests">
                            <xsl:with-param name="suite" select="." />
                            <xsl:with-param name="auth" select="($auth)" />
                            <xsl:with-param name="isFunctional" select="($isFunctional)" />
                        </xsl:call-template>
                    </xsl:variable>
                    <xsl:variable name="failed" select="($totalTests) - ($totalSuccess)"/>
                    <xsl:if test="($failed) > 0 and ($failed_tests_only) = 'false'">
                    <xsl:choose>
                        <xsl:when test="($test_suites_only) = 'false'">    
                            <saxon:assign name="background">error</saxon:assign>
                        </xsl:when>
                        <xsl:otherwise>
                            <saxon:assign name="background">errorsingleborder</saxon:assign>
                        </xsl:otherwise>
                    </xsl:choose>
                    </xsl:if>
                    <xsl:if test="($failed) > 0 or ($failed_tests_only) = 'false'">
                        <tr>
                            <xsl:choose>
                                <xsl:when test="($test_suites_only) = 'false'">
                                    <td align="left" colspan="2" style="padding-left:{$widthPX}">
                                        <xsl:if test="($depth) = 0 and ($isSuite) = 0">
                                            <xsl:attribute name="class"><xsl:value-of select="($background)" /></xsl:attribute>
                                        </xsl:if>
                                        <xsl:if test="($report_format_mode='pdf')">
                                           <xsl:call-template name="make_indentation">
                                              <xsl:with-param name="indentation" select="($depth)*4"/>
                                           </xsl:call-template>
                                        </xsl:if>
                                        <b><xsl:value-of select="($test_name)" /></b>
                                        <font class="gray">
                                            <xsl:text> [ </xsl:text>
                                            <xsl:choose>
                                                <xsl:when test="($isFunctional) = 'true'">
                                                    <xsl:value-of select="($totalSuccess)"/>
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <xsl:value-of select="($totalFailed)"/>
                                                </xsl:otherwise>
                                            </xsl:choose>
                                            <xsl:text> / </xsl:text>
                                            <xsl:value-of select="($totalTests)"/>
                                            <xsl:text> ]</xsl:text>
                                        </font>   
                                    </td>     
                                </xsl:when>
                                <xsl:otherwise>
                                    <td align="left" style="padding-left:{$widthPX}"
                                        class="{$background}">
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <xsl:if test="($report_format_mode='pdf')">
                                           <xsl:call-template name="make_indentation">
                                              <xsl:with-param name="indentation" select="($depth)*4"/>
                                           </xsl:call-template>
                                        </xsl:if>
                                        <xsl:value-of select="($test_name)" />
                                    </td>
                                    <td align="right" class="{$background}">
                                        <xsl:value-of select="($totalFailed)"/>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                    <td align="right" class="{$background}">
                                        <xsl:value-of select="($totalSuccess)"/>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                    <td align="right" class="{$background}">
                                        <xsl:value-of select="($totalTests)"/>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                    <td align="right" class="{$background}">
                                        <xsl:variable name="numerator">
                                            <xsl:choose>
                                                <xsl:when test="($isFunctional) = 'true'">
                                                    <xsl:value-of select="($totalSuccess)" />
                                                </xsl:when>
                                                <xsl:otherwise>
                                                    <xsl:value-of select="($totalFailed)" />
                                                </xsl:otherwise>
                                            </xsl:choose>
                                        </xsl:variable>
                                        <xsl:call-template name="get_percent">
                                            <xsl:with-param name="numerator" select="($numerator)"/>
                                            <xsl:with-param name="denominator" select="($totalTests)"/>
                                        </xsl:call-template>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                    </td>
                                </xsl:otherwise>
                            </xsl:choose>
                        </tr>
                    </xsl:if>
                </xsl:for-each>
            </xsl:when>
            <xsl:otherwise>
                <xsl:if test="($test_suites_only) = 'false'">
                    <xsl:if test="($isFailed) > 0 or ($failed_tests_only) = 'false'">
                        <xsl:variable name="associatedErrors">
                            <xsl:choose>
                                <xsl:when test="($auth) = ''">
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/FunctionalTests/FuncViols/*[@testCaseId=($testID) and not(@supp='true')])" />
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@testCaseId=($testID) and not(@supp='true')])" />
                                        </xsl:otherwise>
                                    </xsl:choose>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:choose>
                                        <xsl:when test="($isFunctional) = 'true'">
                                            <xsl:value-of select="count(/ResultsSession/FunctionalTests/FuncViols/*[@testCaseId=($testID) and @auth=($auth) and not(@supp='true')])" />
                                        </xsl:when>
                                        <xsl:otherwise>
                                            <xsl:value-of select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@testCaseId=($testID) and @auth=($auth) and not(@supp='true')])" />
                                        </xsl:otherwise>
                                    </xsl:choose>
                                    
                                </xsl:otherwise>
                            </xsl:choose>
                        </xsl:variable>
                        <tr>
                            <td align="left" class="{$background}" style="padding-left:{$widthPX}">
                                <xsl:if test="($report_format_mode='pdf')">
                                    <xsl:call-template name="make_indentation">
                                       <xsl:with-param name="indentation" select="($depth)*4"/>
                                    </xsl:call-template>
                                </xsl:if>
                                <xsl:for-each select="($test)">
                                    <xsl:if test="($hasDetails) and (not(($report_type) = ($email_report)))">
                                        <span onclick="toggleTestDetails('{$detailsID}', this)" class="toggle_button" onmouseover="this.style.cursor='pointer'" >[-]</span>
                                    </xsl:if>
                                    <xsl:value-of select="@name" />
                                </xsl:for-each>
                            </td>
                            <td align="right" class="{$background}">
                                <font class="{$fontColor}">
                                    <b><xsl:value-of select="($associatedErrors)" /><xsl:value-of select="($nbsp)" disable-output-escaping="yes"/></b>
                                </font>
                            </td>
                        </tr>
                        <xsl:if test="($hasDetails)">
                            <xsl:call-template name="print_test_summary_details">
                                <xsl:with-param name="comment" select="./@comment" />
                                <xsl:with-param name="screenshots" select=".//Screenshot" />
                                <xsl:with-param name="detailsID" select="($detailsID)" />
                                <xsl:with-param name="depth" select="($depth) + 1" />
                            </xsl:call-template>
                        </xsl:if>
                    </xsl:if>
                </xsl:if>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:if test="($test_suites_only) = 'false'">
            <xsl:for-each select="($test)">
                <xsl:choose>
                    <xsl:when test="$isFunctional = 'true'">
                        <xsl:for-each select="./Test[@total > 0] | ./TestSuite[@total > 0]">
                            <xsl:call-template name="func_test_info">
                                <xsl:with-param name="test" select="." />
                                <xsl:with-param name="testID" select="./@id" />
                                <xsl:with-param name="auth" select="($auth)"/>
                                <xsl:with-param name="depth" select="($depth) + 1" />
                                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:for-each select="./Test[@changeTotal > 0] | ./TestSuite[@changeTotal > 0]">
                            <xsl:call-template name="func_test_info">
                                <xsl:with-param name="test" select="." />
                                <xsl:with-param name="testID" select="./@id" />
                                <xsl:with-param name="auth" select="($auth)"/>
                                <xsl:with-param name="depth" select="($depth) + 1" />
                                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                            </xsl:call-template>
                        </xsl:for-each>
                    </xsl:otherwise>
                </xsl:choose>
            

            </xsl:for-each>
        </xsl:if>
    </xsl:template>

    <!-- ==== print detail information for a single test under the test summary ==== -->
	<xsl:template name="print_test_summary_details">
        <xsl:param name="comment" />
        <xsl:param name="screenshots" />
        <xsl:param name="detailsID" />
        <xsl:param name="depth" />
        
        <xsl:variable name="width" select="($depth) * 20" />
        <xsl:variable name="widthPX" select="concat(($width),'px')" />

		<tr class="details_container" id="{$detailsID}">
			<td colspan="2" class="suite_details">
				<div class="suite_details" style="margin-left:{$widthPX}">

				     <table border="0" cellpadding="0" cellspacing="0">
				        <xsl:if test="($report_format_mode='pdf')">
   				           <xsl:if test="($depth &lt; 2)">
                              <xsl:attribute name="cols">2% 98%</xsl:attribute>                
                           </xsl:if>
   				           <xsl:if test="($depth = 2)">
                              <xsl:attribute name="cols">3% 97%</xsl:attribute>                
                           </xsl:if>
   				           <xsl:if test="($depth = 3)">
                              <xsl:attribute name="cols">4% 96%</xsl:attribute>                
                           </xsl:if>
   				           <xsl:if test="($depth = 4)">
                              <xsl:attribute name="cols">5% 95%</xsl:attribute>                
                           </xsl:if>
   				           <xsl:if test="($depth = 5)">
                              <xsl:attribute name="cols">6% 94%</xsl:attribute>                
                           </xsl:if>
   				           <xsl:if test="($depth > 5)">
                              <xsl:attribute name="cols">7% 93%</xsl:attribute>                
                           </xsl:if>
                        </xsl:if>
  				        <tr>
				        <xsl:if test="($report_format_mode='pdf')">
				           <td></td>
                        </xsl:if>

				        <xsl:if test="not($report_format_mode='pdf')">
					    <xsl:call-template name="print_screenshots">
					       <xsl:with-param name="screenshots" select="($screenshots)"/>
					       <xsl:with-param name="id" select="($detailsID)"/>
  					    </xsl:call-template>
                        </xsl:if>
						<xsl:if test="(($associations = 'true') and (($comment) != '' or count(./assoc) > 0))">
						    <td class="associations">
								<xsl:call-template name="associations"/>								
								<xsl:if test="($comment) != ''">
								  <b><xsl:value-of select="$RES_PARAMETER_COMMENT"/></b><br/>
								  <div class="test_comment"><xsl:value-of select="($comment)"></xsl:value-of></div>
								</xsl:if>
							</td>
						</xsl:if>
					</tr>	
					</table>
				</div>
			</td>
		</tr>
	</xsl:template>
	
	<!-- ================== print the screenshot(s) ================================ -->
	<xsl:template name="print_screenshots">
	   <xsl:param name="screenshots" />
	   <xsl:param name="id" />
	   
	   <xsl:choose>
	       <xsl:when test="count(($screenshots)) > 1">
               <xsl:variable name="screenshotCount" select="count(($screenshots))" />
                   <xsl:variable name="mul_img">multiple_window.png</xsl:variable>
                   <xsl:variable name="img_uri" select="concat(($output_dir),$mul_img)"/>
                   <xsl:variable name="img_url" select="translate($img_uri, '\','/')"/>
                   <xsl:variable name="res_img" saxon:assignable="yes"></xsl:variable>
                   <saxon:assign name="res_img" select="graph:publish($img_url,$mul_img)"/>
	           <xsl:for-each select="(.//Screenshot)[1]"> <!-- put this in a template -->
                   <td>
                       <table>
                           <tr><td>
                                <div onclick="popupScreenshots('{$id}')" onmouseover="this.style.cursor='pointer';">
                                    <img class="screenshot" style="border:0" src="{$mul_img}" />
                                </div>
                           </td></tr>
                           <tr><td>
                                <a class="multiScreenshotLink" href="#" onclick="popupScreenshots('{$id}'); return false;">View screenshots (<xsl:value-of select="($screenshotCount)"/> in total)</a>
                           </td></tr>
                       </table>
                   </td>
               </xsl:for-each>
               <script language="JavaScript">
                   <xsl:for-each select="($screenshots)">
                       <xsl:variable name="width" select="./@width"/>
                       <xsl:variable name="height" select="./@height"/>
                       <xsl:variable name="url" select="./@url"/>
                       <xsl:variable name="iteration" select="./parent::FuncResultDetails/parent::TestCase/@id"/>
                       <xsl:variable name="code">storeScreenshotData(
                            '<xsl:value-of select="($id)"/>',
                            '<xsl:value-of select="($url)"/>',
                            '<xsl:value-of select="($iteration)"/>',
                            <xsl:value-of select="($width)"/>,
                            <xsl:value-of select="($height)"/>);
                       </xsl:variable>
                       <xsl:value-of select="($code)"/>
                   </xsl:for-each>
               </script>
	       </xsl:when>
	       <xsl:otherwise>
	           <xsl:for-each select="($screenshots)">
		           <xsl:variable name="width" select="./@width" />
			       <xsl:variable name="height" select="./@height" />
			       <xsl:variable name="url" select="./@url" />
			       <td>
                       <table>
                           <tr><td>
                                <div onclick="popupScreenshot('{$url}')" onmouseover="this.style.cursor='pointer';">
                                    <img class="screenshot" width="{$width}" height="{$height}" src="{$url}" />
                                </div>
                           </td></tr>
                           <tr><td>
                                <a class="popupScreenshotLink" href="#" onclick="popupScreenshot('{$url}'); return false;">View screenshot</a>
                           </td></tr>
                       </table>
			       </td>
		       </xsl:for-each>
	       </xsl:otherwise>
	   </xsl:choose>
	</xsl:template>

    <!-- ================== format the associations ================================ -->
    <xsl:template name="format_association_into_line">
        <xsl:param name="associations" />
        
        <xsl:variable name="line" select="substring-before($associations, ';')" />
        <xsl:choose>
            <xsl:when test="$line != ''">
	            <xsl:call-template name="format_association_line">
	                <xsl:with-param name="line" select="$line" />
	            </xsl:call-template>
	            <xsl:variable name="next_line" select="substring-after($associations, ';')" />
	            <xsl:if test="$next_line != ''">
	               <xsl:call-template name="format_association_into_line">
	                   <xsl:with-param name="associations" select="$next_line"/>
	               </xsl:call-template>
	            </xsl:if>
            </xsl:when>
            <xsl:when test="$associations != ''">
                <xsl:call-template name="format_association_line">
                    <xsl:with-param name="line" select="$associations" />
                </xsl:call-template>
            </xsl:when>
        </xsl:choose>
    </xsl:template>
    
    <!-- ================== format a single association name/value pair ============ -->
    <xsl:template name="format_association_line">
        <xsl:param name="line" />
        
        <xsl:variable name="name" select="substring-before($line, ':')" />
        <xsl:variable name="value" select="substring-after($line, ':')" />
        <b><xsl:value-of select="$name" /><xsl:text>:</xsl:text></b><xsl:value-of select="$value" /><br/>
    </xsl:template>

    <!-- ================== get percentage value =================================== -->
    <xsl:template name="get_percent">
        <xsl:param name="numerator"/>
        <xsl:param name="denominator"/>
        
        <xsl:choose>
            <xsl:when test="($denominator) = 0">
                <xsl:text>0</xsl:text>
            </xsl:when>
            <xsl:otherwise>
                <xsl:value-of select="floor((($numerator) div ($denominator))*100)"/>
            </xsl:otherwise>
        </xsl:choose>
        <xsl:text>%</xsl:text>
    </xsl:template>

    <!-- =================  print details of functional errors ====================== -->
    <xsl:template name="func_summary_details">
       <xsl:param name="auth"/>
       <xsl:param name="authid"/>
       <xsl:param name="isFunctional"/>

       <xsl:variable name="author_func_history" select="count(/ResultsSession/FunctionalTests/History/Graph[@id=$func_tasks_graph_id]/Authors/Author[@name = $auth]/X)"/>

       <!-- functional history graph -->
       <xsl:if test="($generate_graphs = 'true') and (($author_func_history) > 0) and (($isFunctional) = 'true')">
          <tr><td colspan="3">
          <xsl:call-template name="generate_graph">
             <xsl:with-param name="graph" select="/ResultsSession/FunctionalTests/History/Graph[@id=$func_tasks_graph_id]"/>
             <xsl:with-param name="auth" select="$auth"/>
             <xsl:with-param name="img_name" select="concat('func_',$authid,'_',$org_time)"/>
          </xsl:call-template>
          <br/>
          </td></tr>       
       </xsl:if>
	   <xsl:if test="not($send_links_only_mode = $trends) or not(($report_type) = ($email_report))">
       <xsl:call-template name="func_summary_details_by_test">
          <xsl:with-param name="auth"  select="$auth"/>
          <xsl:with-param name="authid"  select="$authid"/>
          <xsl:with-param name="isFunctional" select="($isFunctional)"/>
       </xsl:call-template>
       </xsl:if>
    </xsl:template>

    <!-- =================  details for each test case ============================== -->
    <xsl:template name="func_test_details">
        <xsl:param name="topTest" />
        <xsl:param name="auth" />
        <xsl:param name="isFunctional"/>
        <xsl:for-each select="$topTest">
            <xsl:variable name="testID" select="@id" />
            <xsl:variable name="associatedErrors" saxon:assignable="yes"/>
            <xsl:choose>
                <xsl:when test="($isFunctional) = 'true'">
                    <saxon:assign name="associatedErrors" select="count(/ResultsSession/FunctionalTests/FuncViols/*[@auth=($auth) and @testCaseId=($testID)])"/>                    
                </xsl:when>
                <xsl:otherwise>
                    <saxon:assign name="associatedErrors" select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@auth=($auth) and @testCaseId=($testID)])"/>
                </xsl:otherwise>
            </xsl:choose>
            <xsl:if test="(($associatedErrors) > 0)">
                <tr>
                    <td style="padding-left:10px;padding-right:10px" colspan="3">
                        <b>
                            <xsl:call-template name="func_test_name">
                                <xsl:with-param name="includePath" select="true()"/>
                                <xsl:with-param name="name" select="@name"/>
                                <xsl:with-param name="testID" select="($testID)"/>
                                <xsl:with-param name="testNode" select="."/>
                            </xsl:call-template>
                        </b>
                    </td>
                </tr>
                <xsl:variable name="allEntries" saxon:assignable="yes" />
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <saxon:assign name="allEntries" select="/ResultsSession/FunctionalTests/FuncViols/*[@auth=($auth) and @testCaseId=($testID)]"/>
                    </xsl:when>
                    <xsl:otherwise>
                        <saxon:assign name="allEntries" select="/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@auth=($auth) and @testCaseId=($testID)]"/>
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:for-each select="($allEntries)">
                    <xsl:variable name="violationId">
                        <xsl:value-of select="($testID)" />:<xsl:value-of select="position()" />
                    </xsl:variable>
                    <xsl:variable name="rule" select="@rule" />
                    <xsl:variable name="sev" select="@sev" />
                    
                    <tr>
                        <td style="padding-left:10px">
                            <xsl:variable name="ln" select="@ln" />
                            <xsl:if test="not($ln = '-1')">
                                <font class="gray">
                                    <xsl:value-of select="($ln)" />:
                                </font>
                            </xsl:if>
                        </td>
                        <td>
                            <xsl:variable name="details" select="@violationDetails"/>
                            <xsl:choose>
                                <xsl:when test="($details) and not(($report_type) = ($email_report))">
                                    <xsl:element name="a">
                                    <xsl:attribute name="href">#</xsl:attribute>
                                    <xsl:attribute saxon:disable-output-escaping="yes" name="onclick">showDetails('<xsl:value-of select="$time"/>', 'details<xsl:value-of select="$violationId"/>')' return false;</xsl:attribute>
 	                                	<xsl:value-of select="@msg" />
 	                                	<xsl:element name="input">
 	                                	<xsl:attribute name="type">hidden</xsl:attribute>
 	                                	<xsl:attribute saxon:disable-output-escaping="yes" name="id">details<xsl:value-of select="$violationId"/></xsl:attribute>
 	                                	<xsl:attribute name="value"><xsl:value-of select="$details"/></xsl:attribute>
                                        </xsl:element>
                                    </xsl:element>
                                    
                                    <xsl:variable name="rev" select="@rev"/>
                                    <xsl:if test="(string-length($rev) > 0) and not($rev = 'unknown.revision')">
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="$RES_PARAMETER_REVISION"/></font>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="($rev)"/></font>
                                    </xsl:if>
                                </xsl:when>
                                <xsl:otherwise>
                                    <xsl:value-of select="@msg" />
                                    <xsl:variable name="rev" select="@rev"/>
                                    <xsl:if test="(string-length($rev) > 0) and not($rev = 'unknown.revision')">
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="$RES_PARAMETER_REVISION"/></font>
                                        <xsl:value-of select="($nbsp)" disable-output-escaping="yes"/>
                                        <font class="gray"><xsl:value-of select="($rev)"/></font>
                                    </xsl:if>
                                </xsl:otherwise>
                            </xsl:choose>
                        </td>
                        <td style="padding-right:10px" align="right">
                            <xsl:variable name="requestTrafficTime" select="@requestTrafficTime" />
                            <xsl:variable name="responseTrafficTime" select="@responseTrafficTime" />
                            <xsl:variable name="requestTrafficBody" select="@requestTrafficBody" />
                            <xsl:variable name="responseTrafficBody" select="@responseTrafficBody" />
                            <xsl:variable name="requestTrafficHeaders" select="@requestTrafficHeaders" />
                            <xsl:variable name="responseTrafficHeaders" select="@responseTrafficHeaders" />
                            <xsl:if test="not(($report_type) = ($email_report)) and ($requestTrafficTime) and ($responseTrafficTime) and ($requestTrafficBody) and ($responseTrafficBody)">
                            	<xsl:element name="a">
                            	<xsl:attribute name="href">#</xsl:attribute>
                            	<xsl:attribute saxon:disable-output-escaping="yes" name="href">showTrafficLog('requestHeader<xsl:value-of select="$violationId"/>', 'requestBody<xsl:value-of select="$violationId"/>', 'responseHeader<xsl:value-of select="$violationId"/>', 'responseBody<xsl:value-of select="$violationId"/>', '<xsl:value-of select="$time"/>', '<xsl:value-of select="$requestTrafficTime"/>', '<xsl:value-of select="$responseTrafficTime"/>'); return false;</xsl:attribute>
                                <xsl:value-of select="($RES_PARAMETER_VIEW_TRAFFIC)"/>
                                </xsl:element>
                                <input type="hidden" id="requestHeader{$violationId}" value="{$requestTrafficHeaders}"/>
                                <input type="hidden" id="requestBody{$violationId}" value="{$requestTrafficBody}"/>
                                <input type="hidden" id="responseHeader{$violationId}" value="{$responseTrafficHeaders}"/>
                                <input type="hidden" id="responseBody{$violationId}" value="{$responseTrafficBody}"/>
                            </xsl:if>
                        </td>                        
                    </tr>
                    <xsl:call-template name="insure_stack_trace"/>
                </xsl:for-each>

            </xsl:if>

            <xsl:for-each select="($topTest)/Test">
                <xsl:call-template name="func_test_details">
                    <xsl:with-param name="topTest" select="." />
                    <xsl:with-param name="auth" select="($auth)" />
                    <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                </xsl:call-template>
            </xsl:for-each>
        </xsl:for-each>
    </xsl:template>

    <!-- =================  func summary_details_by_file ============================ -->
    <xsl:template name="func_summary_details_by_test">
        <xsl:param name="auth" />
        <xsl:param name="authid" />
        <xsl:param name="isFunctional" />

        <xsl:variable name="totErrs" select="tmp" saxon:assignable="yes" />
        <xsl:choose>
            <xsl:when test="($isFunctional) = 'true'">
                <saxon:assign name="totErrs"
                    select="count(/ResultsSession/FunctionalTests/FuncViols/*[@auth=($auth) and not(@supp='true')])" />
            </xsl:when>
            <xsl:otherwise>
                <saxon:assign name="totErrs"
                    select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@auth=($auth) and not(@supp='true')])" />
            </xsl:otherwise>
        </xsl:choose>
        
        <xsl:if test="($totErrs > 0)">                             
	        <tr>
			   	<xsl:variable name="message" select="$RES_PARAMETER_TOTAL_CHANGES" saxon:assignable="yes"/>
			   	<xsl:variable name="anchor" saxon:assignable="yes" select="concat('change_',$authid)"/>    		    				    
			    <xsl:if test="($isFunctional) = 'true'">
			    	<saxon:assign name="anchor" select="concat('func_',$authid)"/>
			    	<saxon:assign name="message" select="$RES_PARAMETER_TOTAL_TASKS"/>
			   	</xsl:if>   			                  
		        <xsl:call-template name="details_table_auth_header">
		    		<xsl:with-param name="auth" select="$auth"/>
		    		<xsl:with-param name="number" select="$totErrs"/>    		
					<xsl:with-param name="message" select="$message"/>
					<xsl:with-param name="colspan" select="2"/>
					<xsl:with-param name="anchor" select="$anchor"/>
		   	  	</xsl:call-template>
	   
		    </tr>                   
        </xsl:if>

        <xsl:variable name="tmp_file_name" select="empty"
            saxon:assignable="yes" />

        <!-- by test suite... -->
        <tr>            
    	    <td align="left" colspan="3" >
   			<table class="doubleborder" bordercolor="{$func_bdrcolor}" aligh="center" width="100%" rules="none" cellpadding='0' cellspacing='0'>
   			<xsl:if test="($report_format_mode='pdf')">                                
                <xsl:attribute name="cols">7% 63% 30%</xsl:attribute>                
            </xsl:if>    	           
        <xsl:for-each select="/ResultsSession/ExecutedTestsDetails//Test">
            <xsl:call-template name="func_test_details">
                <xsl:with-param name="topTest" select="." />
                <xsl:with-param name="auth" select="($auth)" />
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
        </xsl:for-each>
        	</table>
            </td>
        </tr>

        <xsl:if test="($totErrs > 0)">
            <tr>
                <td colspan="3">
                    <br />
                </td>
            </tr>
        </xsl:if>

    </xsl:template>

    <!-- =================  functional tests auth list ============================ -->
    <xsl:template name="func_author_list">    
        <xsl:param name="isFunctional" />
        <xsl:for-each select="/ResultsSession/Authors/Author">
    
            <xsl:call-template name="func_author_info">
                <xsl:with-param name="auth" select="@name"/>
                <xsl:with-param name="authid" select="@id"/>
                <xsl:with-param name="authidx" select="position()"/>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>

       </xsl:for-each> 
    </xsl:template>

    <!-- =================  functional tests auth info ============================ -->
    <xsl:template name="func_author_info">
        <xsl:param name="auth" />
        <xsl:param name="authid" />
        <xsl:param name="authidx" />
        <xsl:param name="isFunctional" />

        <xsl:variable name="author_cs_violations"
            select="count(/ResultsSession/CodingStandards/StdViols/*[@auth=($auth) and not(@supp='true')])" />
        <xsl:variable name="author_change_impacts"
            select="count(/ResultsSession/ChangeAdvisor/ChangeImpacts/*[@auth=($auth) and not(@supp='true')])" />
        <xsl:variable name="author_func_violations"
            select="count(/ResultsSession/FunctionalTests/FuncViols/*[@auth=($auth) and not(@supp='true')])" />
        <xsl:variable name="author_exec_violations"
            select="count(/ResultsSession/Exec/ExecViols/ExecViol[@auth=($auth)])" />
        <xsl:variable name="author_tcases"
            select="count(/ResultsSession/Generation/GenerationInformations/GenerationInfo[@auth=($auth)]/TestCaseList/TestCase)" />
        <xsl:variable name="author_recommended_errors"
            select="count(/ResultsSession/CodingStandards/StdViols/*[@auth=($auth) and not(@supp='true') and @urgent='true'])" />
            
        <xsl:variable name="author_all_tasks"
            select="($author_cs_violations) + ($author_tcases) + ($author_exec_violations) + ($author_func_violations) + ($author_change_impacts)" />

        <xsl:variable name="link_class" saxon:assignable="yes">blue_link</xsl:variable>
        <xsl:variable name="font_class" saxon:assignable="yes">blue</xsl:variable>

        <xsl:if test="(($author_func_violations = 0) and ($isFunctional = 'true')) or (($author_change_impacts = 0) and ($isFunctional = 'false'))">
            <saxon:assign name="link_class">green_link</saxon:assign>
            <saxon:assign name="font_class">green</saxon:assign>
        </xsl:if>
        <tr>
            <td class="singleborder" align="left">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:if test="($author_all_tasks) > 0">
                    <xsl:variable name="prefix_type" saxon:assignable="yes"/>
                    <xsl:choose>
                        <xsl:when test="($isFunctional) = 'true'">
                            <saxon:assign name="prefix_type">func_</saxon:assign>
                        </xsl:when>
                        <xsl:otherwise>
                            <saxon:assign name="prefix_type">change_</saxon:assign>
                        </xsl:otherwise>
                    </xsl:choose>
                    <xsl:if test="(($dev_reports) = 'true') and not(($report_type) = ($email_report))">

                        <xsl:variable name="author_html"
                            select="concat($dev_reports_prefix,$authid,'.html#',$prefix_type,$authid)" />
                        <xsl:variable name="onclick" saxon:assignable="yes"
                            select="concat('openWin(',$qt,$author_html,$qt,')')" />
                        <xsl:if test="($report_type) = ($tcm_report)">
                           <xsl:variable name="dev_rep_name" select="concat($dev_reports_prefix,$authid,'.html')"/>
                           <saxon:assign name="onclick" select="concat('openDevReport(',$qt,$dev_rep_name,$qt,')')"/>
                        </xsl:if>
                        <xsl:call-template name="standard_link_ref">
                            <xsl:with-param name="href"
                                select="'#'" />
                            <xsl:with-param name="onclick"
                                select="($onclick)" />
                            <xsl:with-param name="label"
                                select="($auth)" />
                            <xsl:with-param name="class"
                                select="($link_class)" />
                        </xsl:call-template>
                    </xsl:if>
                    <xsl:if
                        test="(($dev_errors) = 'true') and ((($dev_reports) = 'false') or (($report_type) = ($email_report)))">
                        <a class="{$link_class}" href="#{$prefix_type}{$authid}">
                            <xsl:value-of select="($auth)" />
                        </a>
                    </xsl:if>
                    <xsl:if
                        test="(($dev_errors) = 'false') and (($dev_reports = 'false') or (($report_type) = ($email_report)))">
                        <font class="{$font_class}">
                            <xsl:value-of select="($auth)" />
                        </font>
                    </xsl:if>
                </xsl:if>
                <xsl:if test="not(($author_all_tasks) > 0)">
                    <font class="{$font_class}">
                        <xsl:value-of select="($auth)" />
                    </font>
                </xsl:if>
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td>
            <!-- NYI Functional Suppressions
            <td align="right">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:value-of select="($author_cs_supp)" />
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td> -->
            <!-- NYI Functional Test history
            <xsl:if
                test="count(/ResultsSession/FunctionalTests/History/Author) > 0">
                <td align="right">
                    <xsl:value-of select="($nbsp)"
                        disable-output-escaping="yes" />

                    <xsl:call-template
                        name="format_author_total_delta">
                        <xsl:with-param name="delta"
                            select="/ResultsSession/FunctionalTests/History/Author[@name=($auth)]/Delta" />
                    </xsl:call-template>

                    <xsl:value-of select="($nbsp)"
                        disable-output-escaping="yes" />
                </td>
            </xsl:if> -->
            <td class="singleborder" align="right">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                        <xsl:value-of select="($author_func_violations)" />
                    </xsl:when>
                    <xsl:otherwise>
                        <xsl:value-of select="($author_change_impacts)" />
                    </xsl:otherwise>
                </xsl:choose>
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td>
            <td class="singleborder" align="right">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:choose>
                    <xsl:when test="($isFunctional) = 'true'">
                         <xsl:value-of select="results:getTotalValue(/ResultsSession/ExecutedTestsDetails/Total/@authFail,$authidx)" />
                    </xsl:when>
                    <xsl:otherwise>
                         <xsl:value-of select="results:getTotalValue(/ResultsSession/ExecutedTestsDetails/Total/@authChange,$authidx)" />
                    </xsl:otherwise>
                </xsl:choose>
               
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <!-- xsl:call-template name="total_failed_tests">
                    <xsl:with-param name="auth" select="($auth)"/>
                </xsl:call-template-->
            </td>
            <!-- NYI Recommended errors
            <td align="right">
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
                <xsl:value-of select="($author_recommended_errors)" />
                <xsl:value-of select="($nbsp)"
                    disable-output-escaping="yes" />
            </td> -->
        </tr>
        <!-- TODO remove? /xsl:if -->
    </xsl:template>
    
    <!-- =================  print functional error details for each author ===== -->
    <xsl:template name="func_error_details">    
        <xsl:param name="isFunctional"/>
       <xsl:for-each select="/ResultsSession/Authors/Author">
            <br />
            <table width='{$width}' border='0' bordercolor='{$func_bdrcolor}'
                cellpadding='0' cellspacing='0'>
                <xsl:if test="($report_format_mode='pdf')">                                
                    <xsl:attribute name="cols">33% 33% 34%</xsl:attribute>                
                </xsl:if>
                <xsl:call-template name="func_summary_details_by_test">
                    <xsl:with-param name="auth" select="@name"/>
                    <xsl:with-param name="authid" select="@id"/>
                    <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                </xsl:call-template>
            </table>
       </xsl:for-each> 
    </xsl:template>
    
    <!-- ==================  print result details with traffic ============================ -->
    <xsl:template name="func_test_result_details">
        <xsl:param name="topTest" />
        <xsl:param name="auth" />
        <xsl:variable name="testName" select="$topTest/@name" />
        <xsl:variable name="testId" select="@id" />
        
        <xsl:variable name="iterationCount" select="count(FunctionalResultsDetails/DetailedTestResult/Iteration)" />
        <tr>
            <td colspan="2">
                <xsl:call-template name="func_test_name">
                    <xsl:with-param name="includePath" select="true()"/>
                    <xsl:with-param name="name" select="$testName"/>
                    <xsl:with-param name="testID" select="@id"/>
                    <xsl:with-param name="testNode" select="."/>
                </xsl:call-template>
            </td>
            <td align="right" nowrap="nowrap">
                <xsl:if test="$iterationCount > 0">
                    <a href="#" onclick="showTrafficLogWithIterations('{$testId}', '{$iterationCount}', '{$time}'); return false;">
                        <xsl:value-of select="($RES_PARAMETER_VIEW_TRAFFIC)"/>
                    </a>
                    <xsl:for-each select="FunctionalResultsDetails/DetailedTestResult/Iteration">
                        <xsl:variable name="requestTraffic" select="@request" />
                        <xsl:variable name="responseTraffic" select="@response" />
                        <!-- <input type="hidden" id="iterationId{position()}" value="$iterationId"/> -->
                        <input type="hidden" id="executionTimeIterationId{position()}_{$testId}" value="{@execTime}"/>
                        <input type="hidden" id="requestIterationId{position()}_{$testId}" value="{@request}"/>
                        <input type="hidden" id="responseIterationId{position()}_{$testId}" value="{$responseTraffic}"/>
                    </xsl:for-each>
                </xsl:if>
                <xsl:if test="not($iterationCount > 0)">
                    <xsl:value-of select="($RES_PARAMETER_RESULT_NA)"/>
                </xsl:if>
            </td>
        </tr>
        
    </xsl:template>
    
    <!-- ==================  print functional ============================ -->
    <xsl:template name="display_functional">
        <xsl:param name="isFunctional"/>
    
        <!-- print functional tests header -->
        <xsl:if test="(($functional > 0) and ($isFunctional = 'true')) or (($change_advisor > 0) and ($isFunctional = 'false'))">
            <br/>
            <xsl:call-template name="func_header">
                <xsl:with-param name="isFunctional" select="($isFunctional)" />
            </xsl:call-template>
            
            <xsl:if test="($isFunctional) = 'true'">
                <xsl:if test="($generate_graphs = 'true')">
                   <xsl:if test="($func_tc_history) > 0">
                      <br/>
                      <xsl:call-template name="generate_graph">
                         <xsl:with-param name="graph" select="/ResultsSession/FunctionalTests/History/Graph[@id=$func_testcases_graph_id]"/>
                         <xsl:with-param name="img_name" select="concat('func_tc_',$org_time)"/>
                      </xsl:call-template>
                   </xsl:if>
                   <xsl:if test="($func_history) > 0">
                      <br/>
                      <xsl:call-template name="generate_graph">
                         <xsl:with-param name="graph" select="/ResultsSession/FunctionalTests/History/Graph[@id=$func_tasks_graph_id]"/>
                         <xsl:with-param name="img_name" select="concat('func_',$org_time)"/>
                      </xsl:call-template>    
                   </xsl:if>
                </xsl:if>
            </xsl:if>
            
            
            <!-- funtional tests summary -->
            <br/>
            <xsl:call-template name="func_tests_summary">
                <xsl:with-param name="auth"></xsl:with-param>
                <xsl:with-param name="isFunctional" select="($isFunctional)"/>
            </xsl:call-template>
            
            <xsl:if test="not($send_links_only_mode = $trends) or not(($report_type) = ($email_report))">
            
            <xsl:if test="($authors_details) = 'true'">
            
                <br/>
                <xsl:if test="(($func_errors_nmb > 0) and ($isFunctional = 'true')) or (($change_impacts_nmb > 0) and ($isFunctional = 'false'))">
                    <xsl:call-template name="func_authors_summary">
                        <xsl:with-param name="isFunctional" select="($isFunctional)" />
                    </xsl:call-template>
                </xsl:if>
            
                <!-- print functional errors for each author -->
                <xsl:if test="($dev_errors) = 'true'">
                    <xsl:call-template name="func_error_details">
                        <xsl:with-param name="isFunctional" select="($isFunctional)"/>
                    </xsl:call-template>
                </xsl:if>
            
            </xsl:if>
            
            <xsl:variable name="detailsCount" select="count(/ResultsSession/ExecutedTestsDetails//Test/FunctionalResultsDetails//Iteration)" />
            <xsl:if test="$detailsCount > 0">
                <table width='{$width}' border='0' bordercolor='{$func_bdrcolor}'
                    cellpadding='0' cellspacing='0'>
                    <xsl:if test="($report_format_mode='pdf')">
                        <xsl:attribute name="cols">33% 33% 34%</xsl:attribute>
                    </xsl:if>
                    <tr>
                        <td  colspan="2" bgcolor="{$func_bcolor}">
                            <b><xsl:value-of select="($RES_PARAMETER_TEST_TRAFFIC)"/></b>
                        </td>
                        <td align="right" bgcolor="{$func_bcolor}" nowrap="nowrap">
                            <a class="red_link" href="#top">^</a>
                        </td>
                    </tr>
                    <xsl:for-each select="/ResultsSession/ExecutedTestsDetails//Test">
                        <xsl:call-template name="func_test_result_details">
                            <xsl:with-param name="topTest" select="." />
                            <xsl:with-param name="auth" />
                        </xsl:call-template>
                    </xsl:for-each>
                        
                </table>
            </xsl:if>
            </xsl:if>
        </xsl:if>
    </xsl:template>

</xsl:stylesheet>
